#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 13 14:06:33 2020

@author: isegura
"""


from repartidores import Repartidores
from repartidores import Repartidor

from paquetes import Paquetes
from paquetes import Paquete
from paquetes import PaqueteEntregado
from paquetes import PaqueteIndicencia



import random 


class GestionAmazon:
    """Clase principal para representar la funcionalidad de la fase 1"""
    
    def __init__(self):
        self.paquetes=Paquetes()
        self.repartidores=Repartidores()
        self.entregados=Paquetes()
        self.incidencias=Paquetes()


    def cargarPedidos(self,data):
        """Implementa una función, cargarPedidos(), que reciba los datos 
        necesarios para inicializar el objeto que almacena los pedidos 
        que Amazon deberá enviar."""
        for t in data:
            idpaq=t[0]
            via=t[1]
            num=t[2]
            cp=t[3]
            paquete=Paquete(idpaq,via,num,cp)
            self.paquetes.enqueue(paquete)

    def cargarRepartidores(self,data):
        """Implementa una función, cargarPedidos(), que reciba los datos 
        necesarios para inicializar el objeto que almacena los pedidos 
        que Amazon deberá enviar."""
        for t in data:
            idrep=t[0]
            nombre=t[1]
            estado=t[2]
            zonas=t[3]
                
            repartidor=Repartidor(idrep,nombre,estado,zonas)
            self.repartidores.insertSorted(repartidor)
           
    
    def mostrarRepartidores(self):
        """muestra los repartidores"""
        #print(str(self.repartidores))
        #print()
        return self.repartidores


    def asignarReparto(self):
        """Implementa una función, asignarReparto() que asigne 
        cada pedido (paquete) a un repartidor. Recuerda que el 
        repartidor debe estar activo y cubrir la zona del paquete. 
        Si no hay ningún repartidor activo para esa zona, 
        el paquete (y su zona) será asignado al repartidor 
        activo con menor número de zonas. """
        while not self.paquetes.isEmpty():
            paquete=self.paquetes.dequeue()
            repartidor=self.repartidores.getRepartidorByZona(paquete.cp)
            if repartidor!=None:
                repartidor.asignarPaquete(paquete)
            else:
                minRepartidor=self.repartidores.getRepartidorMin()
                if minRepartidor!=None:
                    minRepartidor.asignarZona(paquete.cp)
                    minRepartidor.asignarPaquete(paquete)
                    
                    
    def reparto(self):
        """Escriba una función, reparto(), 
        que permita simular la entrega de paquetes de todos los repartidores. """   
        for i in range(len(self.repartidores)):
            repartidor=self.repartidores.getAt(i)
            
            if repartidor.estado!='Activo':
               print(repartidor.idrep + ' no está activo\n\n.')
                
            elif len(repartidor.paquetes)==0:
                print(repartidor.idrep + ' no tiene paquetes a repartir.\n\n')
            else:
                self.entregaPaquetes(repartidor)
               
                
    def bajaRepartidor(self,idrep):
        """Escriba una función, bajaRepartidor(), que reciba un repartidor 
        y lo ponga como no activo. Si el repartidor tenía algún paquete por 
        entregar, deberá ser reasignado a algún repartidor activo que cubra 
        la zona del paquete. Si no se encuentra ningún repartidor disponible, 
        el paquete será registrado en incidencias. """
        repartidor=self.repartidores.findRepartidor(idrep)
        if repartidor!=None:
            print('vamos a dar de baja:',idrep)
            repartidor.estado='No activo'
            if len(repartidor.paquetes)>0:
                while not repartidor.paquetes.isEmpty():
                    paq=repartidor.paquetes.dequeue()
                    rep=self.repartidores.getRepartidorByZona(paq.cp)
                    if rep!=None:
                        print('reasignado paquete ', paq.idpaq, ' a:',rep.idrep)
                        rep.asignarPaquete(paq)
                    else:
                        print('no hay repartidor para la zona',paq.cp, ' del paquete:',paq.idpaq)
                        self.incidencias.enqueue(PaqueteIndicencia(paq.idpaq,'','no hay repartidores disponibles.'))
                    
                    
                
    def entregaPaquetes(self,repartidor):
        print('\nprocesando reparto de: ')
        print(str(repartidor))
        
        while not repartidor.paquetes.isEmpty():
            paquete=repartidor.paquetes.dequeue()
            print('Paquete a entregar...',str(paquete))
            #simulamos si se puede entregar o no
            if random.randint(0,1)==1:
                print('Entregado:',paquete.idpaq,' entregado por:',repartidor.idrep,' al intento:',paquete.numIntento)
                self.entregados.enqueue(PaqueteEntregado(repartidor.idrep,paquete.idpaq))
            else:
                paquete.numIntento+=1
                if paquete.numIntento==3:
                    print('Incidencia del paquete:',paquete.idpaq,' repartidor por:',repartidor.idrep,' intentos:',paquete.numIntento)
                    self.incidencias.enqueue(PaqueteIndicencia(paquete.idpaq,repartidor.idrep,'superado número intentos'))
                else:
                    print('No se ha podido entregar:',paquete.idpaq,' repartidor por:',repartidor.idrep,' intentos:',paquete.numIntento)
                    repartidor.paquetes.enqueue(paquete)
                