#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 13 14:57:54 2020

@author: isegura
"""

from dlist import DList
from dlist import DNode
from paquetes import Paquetes

class Zonas(DList):
    def __str__(self):
        """Returns a string with the elems of the list"""
        temp=self.head
        strList=''
        while temp is not None:
          strList+=str(temp.elem).strip()+', '
          temp=temp.next
        if len(strList)>0:
            strList=strList.strip()[:-1]
        return strList


class Repartidor:
    """Clase para represenar un repartidor """
    def __init__(self,idrep,nombre,estado,zonas):
        self.idrep=idrep
        self.nombre=nombre
        self.estado=estado
        self.zonas=Zonas()
        for z in zonas:
            self.zonas.addLast(z)
            
        self.paquetes=Paquetes()
        
        
    def asignarZona(self,zona):
        "asigna una zona al repartidor"
        if self.zonas.index(zona)==-1:
           self.zonas.addLast(zona)
           
       
    def asignarPaquete(self,paquete):
        """almacenar una pquete al final"""
        if not self.paquetes.existe(paquete):
            self.paquetes.enqueue(paquete)
    
    def __str__(self):
        """Devuelve un string con la información del paquete: id del paquete, 
        via, número, código postal y número de intentos"""
        result='Repartidor: '+self.idrep + ", " + self.nombre + ", " +self.estado
        
        if len(self.zonas)>0:
            result+=', Zonas: ' + str(self.zonas)
            
        if len(self.paquetes)>0:
            result+='\n\tPaquetes:\n\t\t'+str(self.paquetes)+'\n'
        
        return result +'\n'  
    
class Repartidores(DList):
    def insertSorted(self,repartidor):
        "inserta los repartidores de forma ordenada por nombre"
        if self.isEmpty():
            self.addLast(repartidor)
            return
        
        node=self.head
        while node and repartidor.nombre>node.elem.nombre:
            node=node.next
            
        if node==self.head:
            self.addFirst(repartidor)
        elif node==None:
            self.addLast(repartidor)
        else:
            newNode=DNode(repartidor)
            prev=node.prev
            newNode.next=node
            newNode.prev=prev
            prev.next=newNode
            node.prev=newNode
            self.size+=1
            
            
    def findRepartidor(self,idrep):
        """busca el repartidor y lo devuelve"""
        node=self.head
        while node:
            if node.elem.idrep==idrep:
                return node.elem
            node=node.next
            
        return None
    
    
    def getRepartidorByZona(self,zona):
        node=self.head
        result=None
        while node and result==None:
            repartidor=node.elem
            if repartidor.estado=='Activo' and repartidor.zonas.index(zona)!=-1:
                result=repartidor
                
            node=node.next
        
        return result
    
    def getRepartidorMin(self):
        minZonas=99999
        index=-1
        for i in range(len(self)):
            rep=self.getAt(i)
            if rep.estado=='Activo' and len(rep.zonas)<minZonas:
                minZonas=len(rep.zonas)
                index=i
        
        if index==-1:
            print('No existe ningún repartidor activo')
            return None
        
        return self.getAt(index)
    
    def __str__(self):
        """Returns a string with the elems of the list"""
        temp=self.head
        strList=''
        while temp is not None:
          strList+=str(temp.elem).strip()+',\n\n'
          temp=temp.next
        if len(strList)>0:
            strList=strList.strip()[:-1]
        return strList
        
    