#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar 25 21:22:26 2020

@author: isegura
"""
from binarytree import Node
from binarysearchtree import BinarySearchTree
from dlist import DList

class Repartidores(BinarySearchTree):
    
    def toList(self):
        l=DList()
        return self._toList(self.root,l)
        
    
    def _toList(self,node,l):
        if node:
            self._toList(node.left,l)
            l.addLast(node.elem)
            self._toList(node.right,l)
            return l
        else:
            return DList()
        

class Zona:
    def __init__(self,cp):
        self.cp=cp
        self.repartidores=Repartidores()
    def __str__(self):
        result=str(self.cp)
        if self.repartidores.root!=None:
            result+=':\t'+str(self.repartidores.toList())
        return result
    
    def __eq__(self,zona):
        return zona!=None and self.cp==zona.cp
    
    def __lt__(self,zona):
        return zona!=None and self.cp<zona.cp
   
    def __gt__(self,zona):
        return zona!=None and self.cp>zona.cp
    

 
    
class Zonas(BinarySearchTree):

    def crearZona(self,cp):
        self.insert(Zona(cp))

    def zonas(self):
        """Devuelve la lista de zonas ordenadas de forma ascendente"""
        lZonas=DList()
        self._zonas(self.root,lZonas)
        return lZonas
        
    
    def _zonas(self,node,lZonas):
        if node:
            self._zonas(node.left,lZonas)
            lZonas.addLast(node.elem)
            self._zonas(node.right,lZonas)
            
       
        
    def asignarRepartidor(self,cp,repartidor):
        """asigna un repartidor a una zona"""
        node=self.find(Zona(cp))
        if node==None:
            #print(cp,' does not exist!!!')
            return
        
        node.elem.repartidores.insert(repartidor)
        
    def borrarRepartidor(self,cp,repartidor):
        """borra un repartidor de una zona"""
        node=self.find(Zona(cp))
        if node==None:
            #print(cp,' does not exist!!!')
            return
        
        node.elem.repartidores.remove(repartidor)

    
    def repartidores(self,cp):
        """Devuelve los repartidores de una zona"""
        node=self.find(Zona(cp))
        if node==None:
            #print(cp,' does not exist!!!')
            return DList()
        
        return node.elem.repartidores.toList()
        
        
    def _predecesoras(self,cp):
        
        pred=DList()
        aux=self.zonas()
        if len(aux)>0:
            pos=aux.index(Zona(cp))
            if pos>0:
                for i in range(0,pos):
                    pred.addFirst(aux.getAt(i).cp)
        return pred
    
    
            
    def _sucesoras(self,cp):
        suc=DList()
        aux=self.zonas()
        if len(aux)>0:
            pos=aux.index(Zona(cp))
            if pos>0:
                for i in range(pos+1,len(aux)):
                    suc.addLast(aux.getAt(i).cp)
        return suc   
        
    
    def _repartirRepartidores(self,lstRep,zonas):
        if len(zonas)==0:
            print('zonas is empty')
            return
        
        i=0
        while lstRep.isEmpty()==False:
            r=lstRep.removeFirst()
            z=zonas.getAt(i)
            #print('reasignando ', r,' a',z)
            
            self.asignarRepartidor(z,r)
            i+=1
            if i==len(zonas):
                i=0
                
    
    def borrarZona(self,cp):
        #print('removing ',cp)

        pred=self._predecesoras(cp)
        #print('predesoras:',str(pred))
        
        suc=self._sucesoras(cp)
        #print('sucesoras:',str(suc))
        
        lstRep=self.repartidores(cp)
        #print('repartidores: ',str(lstRep))

        k=len(lstRep)
        if k>0:
            if len(pred)==0:
                self._repartirRepartidores(lstRep,suc)
            elif len(suc)==0:
                self._repartirRepartidores(lstRep,pred)
            else:
                l1=DList()
                for i in range(k//2):
                    l1.addLast(lstRep.getAt(i))
                    
                #print('l1:',str(l1))
                self._repartirRepartidores(l1,pred)


                
                l2=DList()
                for i in range(k//2,len(lstRep)):
                    l2.addLast(lstRep.getAt(i))
                
                #print('l2:',str(l2))
                self._repartirRepartidores(l2,suc)

        #Finally, we must remove the node
        self.remove(Zona(cp))
        
       
    
        
    def fbalance(self,node):
        """calcula el factor de equilibrio basado en tamnaño"""
        if node==None:
            return 0
        
        sizeL=self._size(node.left)
        sizeR=self._size(node.right)
        return abs(sizeL-sizeR)
    
    def isBalanced(self):
        return self._isBalanced(self.root)
    
    def _isBalanced(self,node):
        """comprueba si un nodo está balanceado"""
        if node==None:
            return True
        
        return self.fbalance(node)<=1 and self._isBalanced(node.left)  and self._isBalanced(node.right)
       
    def balance(self):
       self._balance(self.root)
       
    def _balance(self,node):
        """función para balancear un nodo"""
        while self.fbalance(node)>1:
            #print('balanceando ',node.elem)
            if self._size(node.left)>self._size(node.right):
                
                #copiamos el nodo en el subárbol derecho
                if node.right==None:
                    newNode=Node(node.elem)
                    newNode.parent=node
                    node.right=newNode
                else:
                    self._insertNode(node.right,node.elem)

                #buscamos su predecesor en el subárbol izquierdo
                pred=node.left
                while pred.right!=None:
                    pred=pred.right
                    
                #reemplazamos: print(node.elem,'<-',pred.elem)
                node.elem=pred.elem

                #borramos el  nodo predecesor
                self._removeNode(pred)
                print('remove ',pred.elem)

                
            else:
                #copiamos el nodo en el subárbol izquierdo

                
                if node.left==None:
                    newNode=Node(node.elem)
                    newNode.parent=node
                    node.left=newNode
                else:
                    self._insertNode(node.left,node.elem)

                #buscamos su sucesor en el subárbol derecho
                suc=node.right
                while suc.left!=None:
                    suc=suc.left
                    
                   
                #reemplzamos: print(node.elem,'<-',suc.elem)
                node.elem=suc.elem
                #borramos sucesor 
                self._removeNode(suc)
            
            
        if node!=None:
            self._balance(node.left)
            self._balance(node.right)
        
        

def test():
    objZ=Zonas()
    lZonas=[28225,28054,28440,28369,28165,28009,28001,28547,28889,28847]
    #lZonas=[1,2,3,4,5,6,7]
    #.lZonas=[10,9,8,7,6,5,4,3,2,1]
    for z in lZonas:
        objZ.crearZona(z)

    objZ.draw()
    print()

    l=DList()
    l.addLast(3)
    print(l)
    print()
#    
    print("Zonas:",objZ.zonas())
    print()
#
#
    print('Is balanced:',objZ.isBalanced())
#    print()
#    objZ.balance()
#    print()
#    objZ.draw()
#    print()
#    print('Is balanced:',objZ.isBalanced())
#
#
#    objZ.asignarRepartidor(28225,'Herrero, Maria')
#    objZ.asignarRepartidor(28225,'Segura, Isabel')
#    objZ.asignarRepartidor(28225,'Abad, Juan')
#    
#        
#
#    
#    objZ.asignarRepartidor(28847,'Martinez, Leticia')
#    objZ.asignarRepartidor(28847,'Sanz, Jorge')
#    objZ.asignarRepartidor(28847,'Bono, Mario')
#    
#    
#    objZ.asignarRepartidor(28165,'Merlo, Luis')
#    objZ.asignarRepartidor(28165,'Soria, José')
#    objZ.asignarRepartidor(28165,'Canto, Toni')
#    
#    objZ.asignarRepartidor(28054,'Benitez, Manuel')
#    
#    objZ.asignarRepartidor(28009,'Iglesias, Pablo')
#    objZ.asignarRepartidor(28009,'Echenique, Pablo')
#    
#    objZ.asignarRepartidor(28001,'Casado, Pablo')
#    objZ.asignarRepartidor(28001,'Álvarez de Toledo, Cayetana')
#    
#    objZ.asignarRepartidor(28889,'Montero, Irene')
#    
#    objZ.asignarRepartidor(28547,'Ortega Smith, Javier')
#    objZ.asignarRepartidor(28547,'Abascal, Santiago')
#
#
#    objZ.asignarRepartidor(28440,'Espinosa, Ivan')
#    objZ.asignarRepartidor(28440,'Monasterio, Rocío')
#
#
#    
#    
#
#    print()
#    print(objZ.zonas())
#    print()
#    
#    objZ.draw()
#    
#    print('Is balanced:',objZ.isBalanced())
#    print("Repartidores para la zona 28547:",objZ.repartidores(28547))
#    print("Repartidores para la zona 28001:",objZ.repartidores(28001))
#    print("Repartidores para la zona 28165:",objZ.repartidores(28165))
#    
#
#    objZ.borrarRepartidor(28001,'Casado, Pablo')
#    
#    print("Repartidores para la zona 28001:",objZ.repartidores(28001))
#
#    objZ.borrarRepartidor(28054,'Benitez, Manuel')
#    print("Repartidores para la zona 28054:",objZ.repartidores(28054))
#    objZ.draw()
#
#    objZ.borrarZona(28225)
#    objZ.draw()
#    objZ.borrarZona(28440)
#    objZ.draw()
#
#    print('Is balanced:',objZ.isBalanced())
#    print()
#    objZ.balance()
#    print()
#    objZ.draw()
#    print()
#    print('Is balanced:',objZ.isBalanced())

if __name__ == '__main__':
    test()        
    
