#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Mar 29 20:49:06 2020

@author: isegura
"""
import sys

class PuntoEntrega():
    def __init__(self,via,num=None,cp=None):
        self.via=via
        self.num=num
        self.cp=cp
        
    def __eq__(self,other):
        if other==None:
            return False
        
        return self.via==other.via and self.num==other.num and self.cp==other.cp
    
    def __str__(self):
        result=str(self.via)
        if self.num!=None:
            result+=', '+str(self.num)
        if self.cp!=None:
            result+=', '+str(self.cp)
        return result

class AdjacentVertex:
  def __init__(self,vertex,weight):
    self.vertex=vertex
    self.weight=weight
  
  def __str__(self):
    return '('+str(self.vertex)+','+str(self.weight)+')'
 
class Mapa():
    def __init__(self):
        self.puntosEntrega={}
        self.vertices={}
    
    def anadirPuntoEntrega(self,pto):
        i=len(self.puntosEntrega)
        self.puntosEntrega[i]=pto
        self.vertices[i]=[]
        
    def _getIndice(self,pto):
        for i in self.puntosEntrega.keys():
            if self.puntosEntrega[i]==pto:
                return i
        return -1
 
    def __str__(self):
        result=''
        for i in self.vertices.keys():
            result+=str(self.puntosEntrega[i])+':\n'
            for adj in self.vertices[i]:
                result+='\t'+str(self.puntosEntrega[adj.vertex])+', distancia:'+str(adj.weight)+'\n'
        return result
    
       
    def anadirConexion(self,pto1,pto2,distancia):
        #print('new conexion:',pto1,pto2)
        index1=self._getIndice(pto1)
        index2=self._getIndice(pto2)
        if index1==-1:
            print(pto1,' no existe!')
            return
        if index2==-1:
            print(pto2,' no existe!')
            return 
        #print('adding:',index1,index2,distancia)
        self.vertices[index1].append(AdjacentVertex(index2,distancia))
        #print('adding:',index2,index1,distancia)
        self.vertices[index2].append(AdjacentVertex(index1,distancia))

        
    def estanConectados(self,pto1,pto2):
        index1=self._getIndice(pto1)
        index2=self._getIndice(pto2)
        if index1==-1:
            print(pto1,' no existe!')
            return 0
        if index2==-1:
            print(pto2,' no existe!')
            return 0
        
        for adj in self.vertices[index1]:
            if adj.vertex==index2:
                return adj.weight
        #print(pto1,pto2," no están conectados")
        return 0
            
    def eliminarConexion(self,pto1,pto2):
        index1=self._getIndice(pto1)
        index2=self._getIndice(pto2)
        if index1==-1:
            print(pto1,' no existe!')
            return
        if index2==-1:
            print(pto2,' no existe!')
            return 
        
        for adj in self.vertices[index1]:
            if adj.vertex==index2:
                self.vertices[index1].remove(adj)
                break
                
        for adj in self.vertices[index2]:
            if adj.vertex==index1:
                self.vertices[index2].remove(adj)
                break

    
        

    def generarRuta(self): 
        """This function prints the vertices by dfs algorithm"""
        #print('dfs traversal:')
        # Mark all the vertices as not visited 
        visited = [False] * len(self.vertices)

        ruta=[]
        for v in  self.vertices:
            if visited[v]==False:
                self._dfs(v, visited,ruta)
        
        print()
        return ruta
        
    def _dfs(self, v, visited,ruta): 
        # Mark the current node as visited and print it 
        visited[v] = True
        #print(self.puntosEntrega[v], end = ' ') 
        ruta.append(self.puntosEntrega[v])
        # Recur for all the vertices  adjacent to this vertex 
        for adj in self.vertices[v]: 
          i=adj.vertex
          if visited[i] == False: 
            self._dfs(i, visited,ruta) 
            
            
    
    def printSolution(self,distances,previous,v): 
        """imprime los caminos mínimos desde v"""
        for i in range(len(self.vertices)):
          if distances[i]==sys.maxsize:
            print("There is not path from ",v,' to ',i)
          else: 
            minimum_path=[]
            prev=previous[i]
            while prev!=-1:
              minimum_path.insert(0,self.puntosEntrega[prev])
              prev=previous[prev]
            
            minimum_path.append(self.puntosEntrega[i])  
    
            print('Ruta mínima de:',self.puntosEntrega[v],'->',self.puntosEntrega[i],", distancia", distances[i], ', ruta: ',  end= ' ')
            for x in minimum_path:
                print(x,end= ' ')
            print()
    
    def minDistance(self, distances, visited): 
        """This functions returns the vertex (index) with the mininum distance. To do this, 
        we see in the list distances. We 
        only consider the set of vertices that have not been visited"""
        # Initilaize minimum distance for next node 
        min = sys.maxsize 
    
        #returns the vertex with minimum distance from the non-visited vertices
        for i in range(len(self.vertices)): 
          if distances[i] <= min and visited[i] == False: 
            min = distances[i] 
            min_index = i 
      
        return min_index 
    
    def dijkstra(self, v=0): 
        """"This function takes the index of a delivery point pto and calculates its mininum path 
        to the rest of vertices by using the Dijkstra algoritm. Devuelve una lista con las distancias
        y una lista con los vértices anteriores a uno dado en el camino mínimo"""  
        
        
        #we use a Python list of boolean to save those nodes that have already been visited  
        visited = [False] * len(self.vertices) 
    
        #this list will save the previous vertex 
        previous=[-1]*len(self.vertices) 
    
        #This array will save the accumulate distance from v to each node
        distances = [sys.maxsize]*len(self.vertices) 
        #The distance from v to itself is 0
        distances[v] = 0
    
        for i in range(len(self.vertices)): 
          # Pick the vertex with the minimum distance vertex.
          # u is always equal to v in first iteration 
          u = self.minDistance(distances, visited) 
          # Put the minimum distance vertex in the shotest path tree
          visited[u] = True
          
          # Update distance value of the u's adjacent vertices only if the current  
          # distance is greater than new distance and the vertex in not in the shotest path tree 
          for adj in self.vertices[u]:
            i=adj.vertex
            w=adj.weight
            if visited[i]==False and distances[i]>distances[u]+w:
              distances[i]=distances[u]+w   
              previous[i]=u       
              
        #finally, we print the minimum path from v to the other vertices
        #self.printSolution(distances,previous,v)
        return previous,distances
 
    def rutaMinima(self, start, end):
        """calcula la ruta mínima entre dos puntos de entrega"""
        indexStart=self._getIndice(start)
        if indexStart==-1:
            print(start + " does not exist")
            return None
        indexEnd=self._getIndice(end)
        if indexEnd==-1:
            print(end + " does not exist")
            return None
        
        previous,distances=self.dijkstra(indexStart)
        
        #construimos el camino mínimo
        minimum_path=[]
        prev=previous[indexEnd]
        while prev!=-1:
            minimum_path.insert(0,self.puntosEntrega[prev])
            prev=previous[prev]
            
        minimum_path.append(self.puntosEntrega[indexEnd])
        return minimum_path, distances[indexEnd]
    
        
        
        

def test():
    
    
    
    pA=PuntoEntrega('C/A',1,28921)             #A
    pB=PuntoEntrega('C/B',2,28921)             #B
    pC=PuntoEntrega('C/C',3,28922)             #C
    pD=PuntoEntrega('C/D',4,28922)             #D
    pE=PuntoEntrega('C/E',5,28923)             #E
    pF=PuntoEntrega('C/F',6,28923)             #F
    pG=PuntoEntrega('C/G',6,28923)             #G


    puntos=[pA,pB,pC,pD,pE,pF,pG]
    m=Mapa()
    for p in puntos:
        m.anadirPuntoEntrega(p)
    
    print(m)
    
    m.anadirConexion(pA,pB,8)                    #A,B,8
    m.anadirConexion(pA,pC,9)                    #A,C,9
    m.anadirConexion(pA,pD,14)                   #A,D,14
        
    m.anadirConexion(pB,pE,15)                   #B,E,15
    m.anadirConexion(pB,pF,11)                   #B,F,11
        
    m.anadirConexion(pC,pF,2)                    #C,F,2
        
    m.anadirConexion(pF,pG,8)                    #F,G,8
    m.anadirConexion(pD,pG,2)                    #D,G,2
        
  
    print(m)
    
   # m.modificarConexion(m.puntosEntrega[0],m.puntosEntrega[1],8)#A,B,8
    
    print(pA,pB,' conectados?:',m.estanConectados(pA,pB))
    
    print(pB,pA,' conectados?:',m.estanConectados(pB,pA))
    
    m.eliminarConexion(pA,pB)
    print(m)
    
    m.anadirConexion(pA,pB,8)                    #A,B,8

    
    print('generarRutas:',end=' ')
    ruta=m.generarRuta()
    #print('Ruta:',ruta)
    for r in ruta:
        print(r, end=' ')
    print()
    
    minimum_path,d=m.rutaMinima(pA,pF)
    for p in minimum_path:
        print(p,end=' ')
    print('total distance:',d)

#Descomenar para usarlo en Spyder
if __name__ == '__main__':
    test()